#!/usr/bin/env python

from src import min_grammar, generator, parse, interface, trees, minimal, errors
import sys
from gwriter import template, canonical, primes, meta_info
from gwriter.auto_text import *


auto = auto_eng
curr_output = sys.stdout
    

def try_parse(s):
    if min_grammar.l1_code == 'xxx':
        return s
    else:
        return parse.pf2lf(s,min_grammar.l1).strip()


def try_generate(s):
    if min_grammar.l2_code == 'xxx' or s == '??':
        return s
    else:
        return generator.lf2pf(s,min_grammar.l2).strip()

def translate(s):
    return try_generate(try_parse(s))

def split_pre_sent(s):
    return '',s

def translate_text(t):
    t = t.replace('\\\n',' ')
    text = []
    for sent in t.split('\n'):
        pre_s,s1 = split_pre_sent(sent)
        new_s = dict()
        new_s['pre'] = pre_s
        new_s['sent'] = try_parse(s1)
        text.append(new_s)
     
    text = text_level_process(text,min_grammar.l1,True)
    text = text_level_process(text,min_grammar.l2,False)
     
    translation = ''
    for line in text:
        translation += line['pre'] + try_generate(line['sent']) + '\n'
    return translation


def translate_file(filename):
    f = file(filename,'r')
    t = f.read()
    f.close()
    return translate_text(t)


def apply_text_rule(rule,t1,i,parsing):
    if parsing: sd,sc = rule[0], rule[1]
    elif len(rule) == 4:
        sd,sc = rule[3], rule[2]
    else:
        sd,sc = rule[1], rule[0]
    n1 = trees.match_node(t1[i],sd[0]) 
    n2 = trees_match_node(t[i+1],sd[1])
    if n1 and n2:
        trees.change_node(n1,n2,sc[0],sc[1])
           


def text_level_process(text,rules,parsing = True):
    t = [trees.build_tree(s) for s in text]
    
    rules = min_grammar.l1.custom_lf_txt
    if not parsing: rules.reverse()

    for i in range(0,len(t)) - 1:
        for rule in rules:
            apply_text_rule(rule,t,i,parsing)

    return [trees.linearize(s) for s in t]



def translate_inv(s):
    min_grammar.l1, min_grammar.l2 = min_grammar.l2, min_grammar.l1
    min_grammar.l1_code, min_grammar.l2_code = min_grammar.l2_code, min_grammar.l1_code
    s1 = translate(s)
    min_grammar.l1, min_grammar.l2 = min_grammar.l2, min_grammar.l1
    min_grammar.l1_code, min_grammar.l2_code = min_grammar.l2_code, min_grammar.l1_code
    return s1


def l1(lang):
    try:
      min_grammar.set_grammar(1,lang)
      return True
    except errors.ClineError:
      print >> sys.stderr, 'Grammar "' + lang + '" not found'


def l2(lang):
    try:
      min_grammar.set_grammar(2,lang)
      return True
    except errors.ClineError:
      print >> sys.stderr, 'Grammar "' + lang + '" not found'


def t(s):
    if s <> '':
        print translate(s)
    else:
        while s == '':
            print 'SENT>',
            s = raw_input()
        print translate(s)
    return True

def dump_gloss(l):
    if l == '1' or  l == 'l1':
        g = min_grammar.l1
    else:
        g = min_grammar.l2
    print '========'
    print g.code
    print 'GLOSSARY:'
    print g.gloss
    print'---------'
    print 'EMPTY CATEGORIES: ', g.e_list


def toggle_test_mode(NullArg = True):
    interface.test_mode = not interface.test_mode
    if interface.test_mode:
         minimal.dist_cat = raw_input('Looking for item of category: ')
    else:
         minimal.dist_cat = 'c'


def toggle_write_selection_format(f):
    trees.format = f

def test(NullArg):
    sys.stdout.write('?- ')
    s = sys.stdin.readline()[:-1]
    while s != '' and s != 'q':
        lf = try_parse(s)
##        print 'pre:', lf[0]['pre_sent']
##        print 'parse:', lf[0]['lf']
        print 'parse: ', lf
##        print 'generation:', translate_inv(lf[0]['lf']) + '\n'
        print 'generation:', translate_inv(lf) + '\n'
        sys.stdout.write('?- ')
        s = sys.stdin.readline()[:-1]

  
def export(format = None):
    if format == None:
        print >> sys.stderr, "Format non specified"
    elif not format in min_grammar.export_formats:
        print >> sys.stderr, "Unknown format '" + format + "'"
    else:
        print >> sys.stdout, min_grammar.export_js(min_grammar.l1,format)


##def load_hist(code):
##    if hist.make(code):
##        print '"' + code + '" lexicon built'
##    
##
##def save_hist(code):
##    if hist.save_lex_gl(code):
##        print 'Lexicon saved in file "lang_data\\lexica\\compiled_hist_lex\\nsml_' + code + '".py"'


def set_verbose(null_arg = True):
    interface.verbose = not interface.verbose


def set_output_file(fname):
   global curr_output
   f = file(fname,'w')
   curr_output = f



def write_selection(part):
    global curr_output    
    if min_grammar.l2_code == 'xxx':
       ltab = '{p{90pt}@{ }p{248pt}}'
    else:
       ltab = '{|l|l|}'
    print >> curr_output, '\\begin{tabular}' +ltab + '\n\\hline\n'

    s_list = canonical.sent[part][:]
    if (len(s_list) > 0
       and s_list[0] == False):
       s_list.pop(0)
       interface.test_mode = True
       minimal.dist_cat = s_list.pop(0)
    else:
       interface.test_mode = False

    for s in s_list:
        sent_quot = s.split(' :: ')
        sent = sent_quot[0]
        if len(sent_quot) == 2:
            quot = '(' + sent_quot[1] + ')'
        else:
            quot = ''
        try: 
            LF = try_parse(sent)
            if min_grammar.l2_code <> 'xxx': s1 = try_generate(LF)
            else: s1 = ''
        except Exception: s1,LF = ' *** ', ' *** '
        if trees.format <> 'POSTFIX':
           t = trees.build_tree(LF)
           if trees.format == 'LISP':
              LF = trees.export_tree_lisp(t)
           elif trees.format == 'PROLOG':
              LF = trees.export_tree_prolog(t)
           elif trees.format == 'latex':
              LF = trees.export_tree_latex(t)
        if min_grammar.l2_code == 'xxx': 
            print >> curr_output,  sent + ' & \\footnotesize{' + LF.replace('_','\\_') + '} \\\\'
        else:
            print  >> curr_output, sent + ' & ' + s1 + ' \\\\'
    print  >> curr_output, '\\hline\n\\end{tabular}'
    if curr_output <> sys.stdout:
       curr_output.close()
       curr_output = sys.stdout


def write_text(output,auto,part):
    if auto.text.has_key(part):
        print >> output, auto.text[part] + '\n'
    if (min_grammar.l1.text.has_key(part)
        and min_grammar.l1.text[part] <> ''):
        print >> output, min_grammar.l1.text[part] + '\n'

    
def write_entries(output,auto,part):
    s = ''
    if template.lex_entries.has_key(part):
        for pred in template.lex_entries[part]:
            if min_grammar.l1.lex.has_key(pred):
                s += pred + ' : ' + repr(min_grammar.l1.lex[pred]) + '\n\n'
    if s <> '': print >> output,'==\n' + s + '==\n'
        

    
def write_subpart(output,auto,part):
    for subpart in template.subparts[part]:
        print >> output, '\n' + '@.', auto.titles[subpart] + '.@ '
        if template.subparts.has_key(subpart):
            write_subpart(output,auto,subpart)
        else:

            write_text(output,auto,subpart)
            write_entries(output,auto,subpart)
            print >> output, '\n&'
            
            if (len(canonical.sent[subpart]) > 0
                and canonical.sent[subpart][0] == False):
                canonical.sent[subpart].pop(0)
                interface.test_mode = True
            else:
                interface.test_mode = False

            for s in canonical.sent[subpart]:
                sent_quot = s.split(' :: ')
                sent = sent_quot[0]
                if len(sent_quot) == 2:
                    quot = '(' + sent_quot[1] + ')'
                else:
                    quot = ''
                try: s1 = translate(sent)
                except Exception: s1 = ' *** '
                print >> output, '||@@s ' + sent + ' ' + quot + ' || .|' + s1 + '|.||'
            
def write_meta(output):
    print >> output, ':meta:'
    print >> output, ':title:',min_grammar.l1.title
    print >> output, ':author:',meta_info.author
    print >> output, ':date:',meta_info.date,'\n'
    print >> output, ':toc:\n'
    
    

def write_grammar(filename = None):
    global auto
    if filename: output = file(filename,'a')
    else: output = sys.stdout
    save_test_mode, interface.test_mode = interface.test_mode, True

    write_meta(output)
    for part in template.lparts:
        print >> output, '\n\n---+', auto.titles[part],'\n'
        write_text(output,auto,part)
        write_entries(output,auto,part)
        write_subpart(output,auto,part)
    print >> output,'\n:index:\n'
    interface.test_mode = save_test_mode
        
    
commands = {'l1' : l1, 'l2' : l2, 't': t, 'test' : test, 'export' : export,
          #  'lh' : load_hist, 'sh' : save_hist,
            'v': set_verbose, 'gl' : dump_gloss, 'tm' : toggle_test_mode, 'o' : set_output_file,
            'wg': write_grammar, 'ws' : write_selection, 'wsf' : toggle_write_selection_format,
            'tf' : translate_file}


def read_eval_print():
    while True:
        sys.stdout.write('>> ')
        i = sys.stdin.readline()[:-1]
        if i == 'q' or i == 'quit' or i ==  'exit' or i == '': return
        n=i.find(' ')
        if n != -1:
            c,p = i[:n], i[n+1:]
        else:
            c,p = i,''
        if commands.has_key(c):
            commands[c](p)
##            try:
##                commands[c](p)
##            except errors.ClineError, e:
##                print >> sys.stderr, e
        else:
            print >> sys.stderr, "Command not found"




if __name__ == '__main__':
    min_grammar.set_grammar(1,'xxx')
    min_grammar.set_grammar(2,'xxx')
    read_eval_print()
